@@ -9,29 +9,56 @@ import sys
 from os import path
 import tempfile
 from threading import Thread
+from threading import Event
+
+
+def __create_process(server_path, agent_path, timeout, debug):
+
+    if debug:
+        popen_params = ["php", "-f", server_path, agent_path, timeout, 'debug']
+    else:
+        popen_params = ["php", "-f", server_path, agent_path, timeout]
+
+    p = Popen(popen_params,
+                       stdout=subprocess.PIPE,
+                       stderr=subprocess.PIPE)
+    p.stdout.readline()
+
+    if not debug:
+        return
 
-def execute(process):
-    # Poll process for new output until finished
     while True:
-        nextLine = process.stdout.readline()
-        if nextLine == '':
+        nextLine = p.stdout.readline()
+        if nextLine is None or nextLine == '':
             break
         sys.stdout.write(nextLine)
 
 
+def __start_server_process(server_path, agent_path, timeout, event, debug):
+    __create_process(server_path, agent_path, timeout, debug)
+    event.set()
+
+
+def start_and_wait_server(agent_server, agent_path, timeout, debug):
+    event = Event()
+    thread = Thread(target=__start_server_process, args=(agent_server, agent_path, timeout, event, debug))
+    thread.start()
+    event.wait(5)
+
+
 class AgentConnection:
 
     AUTOREPLY_NONE = 1
     AUTOREPLY_DELIVERED = 2
     AUTOREPLY_READ = 3
 
-    def __getAgentPath (self, phoneNumber):
+    def __getAgentPath(self, phoneNumber):
         agentpath = path.join (tempfile.gettempdir(), phoneNumber)
         if not os.path.exists(agentpath):
             os.makedirs(agentpath)
         return agentpath
 
-    def __currentServer (self, serverSocket):
+    def __currentServer(self, serverSocket):
         conn = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
         err = conn.connect_ex (serverSocket)
         if err != 0:
@@ -50,35 +77,27 @@ class AgentConnection:
         currentScriptDir = os.path.dirname(__file__)
         return os.path.join(currentScriptDir,'php','agent-server')
 
-    def __readSubprocessStdout (self,process):
-        thread = Thread(target = execute, args = (process, ))
-        thread.start()
-
-    def __startServer (self, agentpath, timeout):
+    def __startServer(self, agent_path, timeout, debug):
         agent_server = self.__getServerPath()
-        p = Popen (["php","-f", agent_server, agentpath, timeout],
-                           stdout=subprocess.PIPE,
-                           stderr=subprocess.PIPE)
-        firstLine = p.stdout.readline().strip()
-        return firstLine
+        start_and_wait_server(agent_server, agent_path, timeout, debug)
 
-    def __getOrCreateConnection (self, phoneNumber, timeout):
-        agentPath = self.__getAgentPath(phoneNumber)
-        serverSocket = path.join (agentPath, "server.sock")
+    def __getOrCreateConnection(self, phoneNumber, timeout, debug):
+        agent_path = self.__getAgentPath(phoneNumber)
+        serverSocket = path.join (agent_path, "server.sock")
         conn = None
         tries = 0
-        while conn == None or tries < 2:
+        while conn is None and tries < 3:
             conn = self.__currentServer (serverSocket)
-            if conn != None:
+            if conn is not None:
                 return conn
-            self.__startServer (agentPath, timeout)
+            self.__startServer (agent_path, timeout, debug)
             tries = tries + 1
         return conn
 
-
-    def __init__(self, phoneNumber, timeout = "60"):
-        self.conn = self.__getOrCreateConnection (phoneNumber, timeout)
-        self.fd = self.conn.makefile()
+    def __init__(self, phoneNumber, timeout = "120", debug = False):
+        self.conn = self.__getOrCreateConnection (phoneNumber, timeout, debug)
+        if self.conn is not None:
+            self.fd = self.conn.makefile()
 
     def __WriteInt (self, data):
         self.__WriteLine(str(data))
@@ -171,14 +190,14 @@ class AgentConnection:
         self.__WriteLine (number)
         self.__WriteLine (nickname)
         self.__WriteInt (autoreply)
-        return self.__ReadLine ()
+        return self.__ReadObject()
 
     def connect (self, number, nickname, autoreply = AUTOREPLY_DELIVERED):
         self.__WriteLine ('connect')
         self.__WriteLine (number)
         self.__WriteLine (nickname)
         self.__WriteInt (autoreply)
-        return self.__ReadLine ()
+        return self.__ReadObject()
 
     def login (self, password):
         self.__WriteLine ('login')
@@ -188,7 +207,7 @@ class AgentConnection:
 
     def __ReadEvent(self):
         name = self.__ReadLine ()
-        if name == 'OK':
+        if name == 'OK' or name == 'ERROR':
             return None
         data = self.__ReadObject ()
         return { "name": name, "data": data }
@@ -198,8 +217,12 @@ class AgentConnection:
         self.__WriteLine (peekEventName)
         event = self.__ReadEvent ()
         while not (event is None):
+            data = event.get ('data')
+            if data is not None and data.get('code') is not None and data.get('code') == '500':
+                break
             events.append (event)
             event = self.__ReadEvent ()
+
         return events
 
     def peekEvents (self):
@@ -222,10 +245,24 @@ class AgentConnection:
             return {"code":"200"}
         return {"code":"500"}
 
+    def setConnectedStatus (self, status):
+        if status == 'online':
+            return self.sendActiveStatus()
+        return self.sendOfflineStatus()
+
+    def getConnectedStatus (self):
+        self.__WriteLine ('getconnectedstatus')
+        return self.__ReadObject ()
+
     def sendGetGroups(self):
         self.__WriteLine ('sendgetgroups')
         return self.__ReadObject ()
 
+    def sendGetGroupInfo(self, group_id):
+        self.__WriteLine ('sendgetgroupinfo')
+        self.__WriteLine (group_id)
+        return self.__ReadObject ()
+
     def sendGetClientConfig(self):
         self.__WriteLine ('sendgetclientconfig')
         return self.__ReadObject ()
@@ -285,6 +322,17 @@ class AgentConnection:
         self.__WriteLine (subject)
         return self.__ReadObject ()
 
+    def sendSetGroupPicture(self, group_id, path):
+        self.__WriteLine ('sendsetgrouppicture')
+        self.__WriteLine (group_id)
+        self.__WriteLine (path)
+        return self.__ReadObject ()
+
+    def sendRemoveGroupPicture(self, group_id):
+        self.__WriteLine('sendremovegrouppicture')
+        self.__WriteLine(group_id)
+        return self.__ReadObject()
+
     def sendGroupsLeave(self, gjid):
         self.__WriteLine ('sendgroupsleave')
         self.__WriteLine (gjid)
@@ -329,53 +377,77 @@ class AgentConnection:
         self.__WriteLine ('sendmessagedelivered')
         self.__WriteLine (to)
         self.__WriteLine (msgid)
-        return self.__ReadLine ()
+        result = self.__ReadLine()
+        if (result == 'OK'):
+            return {"code":"200"}
+        return {"code":"500"}
 
     def sendMessageDeliveredBatch(self, to, msgids):
         self.__WriteLine ('sendmessagedeliveredbatch')
         self.__WriteLine (to)
         self.__WriteArray (msgids)
-        return self.__ReadLine ()
+        result = self.__ReadLine()
+        if (result == 'OK'):
+            return {"code":"200"}
+        return {"code":"500"}
 
     def sendMessageRead(self, to, msgid):
         self.__WriteLine ('sendmessageread')
         self.__WriteLine (to)
         self.__WriteLine (msgid)
-        return self.__ReadLine ()
+        result = self.__ReadLine()
+        if (result == 'OK'):
+            return {"code":"200"}
+        return {"code":"500"}
 
     def sendMessageReadBatch(self, to, msgids):
         self.__WriteLine ('sendmessagereadbatch')
         self.__WriteLine (to)
         self.__WriteArray (msgids)
-        return self.__ReadLine ()
+        result = self.__ReadLine()
+        if (result == 'OK'):
+            return {"code":"200"}
+        return {"code":"500"}
 
     def sendGroupMessageRead(self, to, msgid, participant):
         self.__WriteLine ('sendgroupmessageread')
         self.__WriteLine (to)
         self.__WriteLine (msgid)
         self.__WriteLine (participant)
-        return self.__ReadLine ()
+        result = self.__ReadLine()
+        if (result == 'OK'):
+            return {"code":"200"}
+        return {"code":"500"}
 
     def sendGroupMessageReadBatch(self, to, msgids, participant):
         self.__WriteLine ('sendgroupmessagereadbatch')
         self.__WriteLine (to)
         self.__WriteArray (msgids)
         self.__WriteLine (participant)
-        return self.__ReadLine ()
+        result = self.__ReadLine()
+        if (result == 'OK'):
+            return {"code":"200"}
+        return {"code":"500"}
 
     def sendGroupMessageDelivered(self, to, msgid, participant):
         self.__WriteLine ('sendgroupmessagedelivered')
         self.__WriteLine (to)
         self.__WriteLine (msgid)
         self.__WriteLine (participant)
-        return self.__ReadLine ()
+        result = self.__ReadLine()
+        if (result == 'OK'):
+            return {"code":"200"}
+        return {"code":"500"}
 
     def sendGroupMessageDeliveredBatch(self, to, msgids, participant):
         self.__WriteLine ('sendgroupmessagedeliveredbatch')
         self.__WriteLine (to)
         self.__WriteArray (msgids)
         self.__WriteLine (participant)
-        return self.__ReadLine ()
+        result = self.__ReadLine()
+        if (result == 'OK'):
+            return {"code":"200"}
+        return {"code":"500"}
 
     def sendMessageAudio(self, to, imageURL, voice = False, msgid = None, storeURLmedia = False, file_size = 0, file_hash = ''):
         self.__WriteLine ('sendmessageaudio')
@@ -391,12 +463,18 @@ class AgentConnection:
     def sendMessageComposing(self, to):
         self.__WriteLine ('sendmessagecomposing')
         self.__WriteLine (to)
-        return self.__ReadLine ()
+        result = self.__ReadLine()
+        if (result == 'OK'):
+            return {"code":"200"}
+        return {"code":"500"}
 
     def sendMessagePaused(self, to):
         self.__WriteLine ('sendmessagepaused')
         self.__WriteLine (to)
-        return self.__ReadLine ()
+        result = self.__ReadLine()
+        if (result == 'OK'):
+            return {"code":"200"}
+        return {"code":"500"}
 
     def sendMessageImage(self, to, imageURL, caption = '', message_id = None, storeURLmedia = False, file_size = 0, file_hash = None):
         self.__WriteLine ('sendmessageimage')
@@ -432,10 +510,13 @@ class AgentConnection:
         self.__WriteLine (file_hash)
         return self.__ReadObject ()
 
-    def sendAvailableForChat(self, nickname):
-        self.__WriteLine ('sendavailableforchat')
+    def sendUpdateNickname(self, nickname):
+        self.__WriteLine ('sendupdatenickname')
         self.__WriteLine (nickname)
-        return self.__ReadLine ()
+        result = self.__ReadLine()
+        if (result == 'OK'):
+            return {"code":"200"}
+        return {"code":"500"}
 
     def sendGetPresences(self, to):
         self.__WriteLine ('sendgetpresences')
@@ -449,7 +530,10 @@ class AgentConnection:
     def sendSetPrivacyBlockedList(self, blockedJids):
         self.__WriteLine ('sendsetprivacyblockedlist')
         self.__WriteArray (blockedJids)
-        return self.__ReadLine ()
+        result = self.__ReadLine()
+        if (result == 'OK'):
+            return {"code":"200"}
+        return {"code":"500"}
 
     def sendSetProfilePicture(self, path):
         self.__WriteLine ('sendsetprofilepicture')
