@@ -30,10 +30,6 @@ class WagentProt extends WhatsProt
         return $this->phoneNumber;
     }
 
-    public function getIdentity ()
-    {
-        return $this->identity;
-    }
 
     public function getSocket ()
     {
@@ -48,4 +44,18 @@ class WagentProt extends WhatsProt
     }
 }
 
+class WagentRegistration extends Registration
+{
+    public function __construct($number, $debug = false, $customPath = false)
+    {
+        parent::__construct($number, $debug, $customPath);
+    }
+
+    public function getIdentity ()
+    {
+        return $this->identity;
+    }
+
+}
+
 ?>
\ No newline at end of file
